basic = require("lib.basic")
networking = require("lib.networking")

oreList = {
  [0] = "minecraft:gold_ore",
  [1] = "minecraft:deepslate_gold_ore",
  [2] = "minecraft:iron_ore",
  [3] = "minecraft:deepslate_iron_ore",
  [4] = "minecraft:coal_ore",
  [5] = "minecraft:deepslate_coal_ore",
  [6] = "minecraft:lapis_ore",
  [7] = "minecraft:deepslate_lapis_ore",
  [8] = "minecraft:diamond_ore",
  [9] = "minecraft:deepslate_diamond_ore",
  [10] = "minecraft:redstone_ore",
  [11] = "minecraft:deepslate_redstone_ore",
  [12] = "minecraft:emerald_ore",
  [13] = "minecraft:deepslate_emerald_ore",
  [14] = "minecraft:nether_quartz_ore",
  [15] = "minecraft:nether_gold_ore",
  [16] = "minecraft:copper_ore",
  [17] = "minecraft:deepslate_copper_ore",
  [18] = "minecraft:ancient_debris",
}

keepList = {
  [0] = "computercraft:wireless_modem_advanced",
  [1] = "minecraft:chest",
  [2] = "minecraft:barrel",
  [3] = "minecraft:raw_gold",
  [4] = "minecraft:raw_iron",
  [5] = "minecraft:coal",
  [6] = "minecraft:lapis_lazuli",
  [7] = "minecraft:diamond",
  [8] = "minecraft:redstone",
  [9] = "minecraft:emerald",
  [10] = "minecraft:quartz",
  [11] = "minecraft:gold_nugget",
  [12] = "minecraft:ancient_debris",
}



local function setState()
  basic.writeToFile("brulijam/files/state-turtle-miner-fibo.txt", textutils.serialize(state))
end

local function getState()
  if not fs.exists "brulijam/files/state-turtle-miner-fibo.txt" then
    turtleState = {}
    turtleState["script"] = "brulijam/programs/turtle-miner-fibo.lua"
    turtleState["info"] = ""
    turtleState["fuelLevel"] = turtle.getFuelLevel()
    turtleState["dataCurrentFibo"] = 2
    turtleState["dataGoalRows"] = 0
    turtleState["dataGoalPathProgress"] = 0
    turtleState["dataCurrentRow"] = 0
    turtleState["dataCurrentPathProgress"] = 0
    basic.writeToFile("brulijam/files/state-turtle-miner-fibo.txt", textutils.serialize(turtleState))
  else
    turtleState = basic.readFileAll("brulijam/files/state-turtle-miner-fibo.txt")
    turtleState = textutils.unserialize(turtleState)
  end
  return turtleState
end


local function savePos(prefix)
  --swap modem with Pickaxe
  --by convention the chunkloader is always left, so that Tools are right
  for i=1, 16 do
    turtle.select(i)
    itemDetail = turtle.getItemDetail()

    if itemDetail ~= nil then
      if itemDetail.name == "computercraft:wireless_modem_advanced" then
        turtle.equipRight()
        posZ, posY, posX = gps.locate()
        sleep(1)
        if prefix == "Chest placed at" then
          posY = posY + 1
        end
        basic.appendToFileNl("/brulijam/files/chests-turtle-miner-fibo.txt", prefix .. posX .. ", " .. posY .. ", " .. posZ)
        networking.sendLog("all", "/brulijam/logs/chests-to-collect.txt", prefix .. posX .. ", " .. posY .. ", " .. posZ)
        turtle.equipRight()
      end
    end
  end
end

local function inventoryRework()
  chestAvailable = false
  chestSlot = 0
  spaceAvailable = false

  for i=1, 16 do
    turtle.select(i)
    itemDetail = turtle.getItemDetail()
    if itemDetail ~= nil then
      if itemDetail.name == "minecraft:coal" then
        turtle.refuel()
        spaceAvailable = true
      elseif itemDetail.name == "minecraft:chest" or itemDetail.name == "minecraft:barrel" then
        chestAvailable = true
        chestSlot = i
      else --check if item can be thrown away
        keepItem = false
        for j=0, #keepList do
          if itemDetail.name == keepList[j] then
            keepItem = true
          end
        end
        if not keepItem then
          turtle.dropDown()
          spaceAvailable = true
        end
      end
    else --at least one slot of space remaining
      spaceAvailable = true
    end
  end

  --store items away
  if not spaceAvailable and chestAvailable then
    turtle.up()
    basic.secureDigUp()
    turtle.up()
    basic.secureDigUp()
    turtle.select(chestSlot)
    turtle.placeUp()
    savePos("Chest placed at")

    for k=1, 16 do
      turtle.select(k)
      itemDetail = turtle.getItemDetail()
      if itemDetail ~= nil then
        if itemDetail.name ~= "minecraft:chest" and itemDetail.name ~= "minecraft:barrel" and itemDetail.name ~= "computercraft:wireless_modem_advanced" then
          turtle.dropUp()
        end
      end
    end
    turtle.down()
    turtle.down()
  end

  if not spaceAvailable and not chestAvailable then
    savePos("Turtle stopped")
    shell.run("delete /brulijam/task.lua")
    shell.run("reboot")
  end
end


local function isOre(blockData)
  for i=0, #oreList do
    if blockData["name"] == oreList[i] then
      inventoryRework()
      return true
    end
  end
  return false
end

local function recursiveOreMining()
  for i=0, 3 do
    turtle.turnLeft()
    flockFound, blockData = turtle.inspect()
    if isOre(blockData) then
      basic.secureDig()
      turtle.forward()
      recursiveOreMining()
      turtle.back()
    end
  end

  flockFound, blockData = turtle.inspectUp()
  if isOre(blockData) then
    basic.secureDigUp()
    turtle.up()
    recursiveOreMining()
    turtle.down()
  end

  flockFound, blockData = turtle.inspectDown()
  if isOre(blockData) then
    basic.secureDigDown()
    turtle.down()
    recursiveOreMining()
    turtle.up()
  end
end

local function mineRow()
  --mine a 3 Block high Tunnel for <distance> Blocks
  for i=state["dataCurrentPathProgress"], state["dataGoalPathProgress"]-2 do
    print("row: " .. state["dataCurrentRow"]+1 .. "/" .. state["dataGoalRows"] .. ", pathProgress: " .. state["dataCurrentPathProgress"]+1 .. "/" .. state["dataGoalPathProgress"])
    
    basic.secureDig()
    turtle.forward()
    basic.secureDigUp()
    basic.secureDigDown()

    --clearInventory()
    --inventoryRework()

    --check current mining step for ore
    turtle.up()
    recursiveOreMining()
    turtle.down()
    recursiveOreMining()
    turtle.down()
    recursiveOreMining()
    turtle.up()
    
    inventoryRework()
    --clearInventory()
    --checkInventory()
    
    --save path progress
    state["dataCurrentPathProgress"] = i+1
    state["fuelLevel"] = turtle.getFuelLevel()
    setState(state)
  end
  print("row: " .. state["dataCurrentRow"]+1 .. "/" .. state["dataGoalRows"] .. ", pathProgress: " .. state["dataCurrentPathProgress"]+1 .. "/" .. state["dataGoalPathProgress"])
end

local function mineChunk()
  state = getState()

  state["dataGoalRows"] = state["dataCurrentFibo"]
  state["dataGoalPathProgress"] = 3*(state["dataCurrentFibo"]-1)+1
  setState(state)

  for i=state["dataCurrentRow"], state["dataGoalRows"]-1 do
    sleep(0.5)
    mineRow(state["dataGoalPathProgress"])
    
    --turn back and return to start of row if not last row
    if i<state["dataGoalRows"]-1 then
      turtle.turnLeft()
      turtle.turnLeft()
      --while not turtle.detect() do
      --  turtle.forward()
      --end

      for j=1, state["dataGoalPathProgress"]-1 do
        turtle.forward()
      end

      --go to next row
      turtle.turnLeft()
      for j=0, 2 do
        basic.secureDig()
        turtle.forward()
        basic.secureDigUp()
        basic.secureDigDown()
      end
      turtle.turnLeft()
    end
    



    state["dataCurrentRow"] = i+1
    state["dataCurrentPathProgress"] = 0
  end

  --reset row progress
  state["dataCurrentRow"] = 0
end

local function mineFibo()
  state = getState()

  while true do
    mineChunk()

    --go to next chunk
    turtle.turnRight()
    for j=1, 2 do
      basic.secureDig()
      turtle.forward()
      basic.secureDigUp()
      basic.secureDigDown()
    end

    --get next fibo number
    state["dataCurrentFibo"] = math.floor((state["dataCurrentFibo"] * (1+math.sqrt(5))/2) + 0.5)
    setState(state)
  end
end


--setup()
mineFibo()